@@ -1,12 +1,14 @@
 from .login_bruteforce import LoginBruteforceTTP
 from .sql_injection import InputFieldInjector, URLManipulation
-from .uuid_guessing import UUIDGuessingTTP
+from .uuid_guessing import GuessUUIDInURL
 from .request_flooding import RequestFloodingTTP
+from .csrf_validation import CSRFValidationTTP
 
 __all__ = [
     'LoginBruteforceTTP',
-    'InputFieldInjector', 
+    'InputFieldInjector',
     'URLManipulation',
-    'UUIDGuessingTTP',
-    'RequestFloodingTTP'
-]
\ No newline at end of file
+    'GuessUUIDInURL',
+    'RequestFloodingTTP',
+    'CSRFValidationTTP'
+]
